#define _GNU_SOURCE

#include <stdio.h>
#include <stdarg.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <openssl/rand.h>

#include "util.h"

/*
 * Logging function
 */
void xlog(const char *format, ...)
{
	va_list ap;
	char line[512];

	va_start(ap, format);
	vsnprintf(line, sizeof(line), format, ap);
	va_end(ap);

	printf("%s", line);
}

/*
 * Logging function
 */
void xerr(const char *format, ...)
{
	va_list ap;
	char line[512];

	va_start(ap, format);
	vsnprintf(line, sizeof(line), format, ap);
	va_end(ap);

	fprintf(stderr, "%s", line);
}

void dump(const unsigned char *d, const unsigned int len)
{
	unsigned int i;

	xlog("Dump:");
	for (i = 0; i < len; i++) {
		if (i % 16 == 0)
			xlog("\t");
		xlog(" %02hhx", d[i]);
		if (i % 16 == 15)
			xlog("\n");
	}
	xlog("\n");
}

/*
 * Reads a password from terminal
 */
int read_password(const char *msg, char *pass, const size_t pass_size)
{
	struct termios old, new;
	int r;

	printf("%s", msg); fflush(stdout);

	r = tcgetattr(0, &old);
	if (r == -1) {
		fprintf(stderr, "Cannot call tcgetattr: %s\n", strerror(errno));
		return -1;
	}

	new = old;
	new.c_lflag &= ~ECHO;
	r = tcsetattr(0, TCSAFLUSH, &new);
	if (r == -1) {
		fprintf(stderr, "Cannot call tcsetattr: %s\n", strerror(errno));
		return -1;
	}

	while (1) {
		r = read(0, pass, pass_size - 1);
		if (r == -1) {
			fprintf(stderr, "Cannot read password: %s\n", strerror(errno));
			break;
		}

		while ((r > 0) && (strchr("\r\n \t", pass[r - 1]))) {
			pass[r - 1] = '\0';
			r--;
		}

		break;
	}
	tcsetattr(0, TCSAFLUSH, &old);

	printf("\n");

        return r;
}

/*
 * Returns @len random bytes
 * getrandom is not supported in CentOS7!
 */
int my_getrandom(void *buf, const size_t len,
	const unsigned int flags __attribute__((unused)))
{
	int ret;

	ret = RAND_bytes(buf, len);
	if (ret == -1)
		return ret;

	return len;
}

